var Macarona = Macarona || {};


const old_gameparty_init = Game_Party.prototype.initialize;
const old_battlemanager_start = BattleManager.startBattle;
const old_gameaction_apply = Game_Action.prototype.apply;
const old_party_createcommandsprites = Window_PartyCommand.prototype.createCommandSprites;

Window_PartyCommand.prototype.createCommandSprites = function() {
  old_party_createcommandsprites.call(this);
  if ($gameSwitches._lucilleKidnapped) {
    if ($modLoader.knownMods.has("absentmind")) {
      var extrabitmap = ImageManager.loadSystem("absent_lucille_command");
    } else {
      var extrabitmap = ImageManager.loadSystem("lucille_command");
    }
    extrabitmap.addLoadListener(() => {
      var sprite = new Sprite(extrabitmap);
      sprite.setFrame(0, 0, 360, 55);
      sprite.y = (2 * (55 + 3)) - 30;
      this._commandSprites[2] = sprite;
      this.addChildToBack(sprite);
      this._frameAnimations = [
      {sprite: sprite, rect: new Rectangle(0, 0, 360, 55), frames: [0, 1, 2, 3], frameIndex: 0, delayCount:  0, delay: 12, active: true}
    ]
    });
  }
}


BattleManager.restartTurn = function() {
  this._phase = "turnEnd";
  this._preemptive = false;
  this._surprise = false;
  if (this.isForcedTurn()) {
    this._turnForced = false;
  }
  this._actorCommandIndex = 0;
}

// COMMON EVENT SKILL KILL ME
Scene_Battle.prototype.commandCallLucille = function() { 
  $gameParty.lucilleEnergy -= 80;
  this.hidePartyCommand();
  $gameTemp.reserveCommonEvent(MacaronaIds.event["Lucille Command"]);
  $gameTemp._callTurnCount = $gameTroop.turnCount();
  BattleManager.restartTurn();
}

Window_PartyCommand.prototype.makeCommandList = function() {
  this.addCommand(TextManager.fight,  'fight');
  if ($gameVariables.value(22) !== 5) {
    this.addCommand(TextManager.escape, 'escape', BattleManager.canEscape());
    if ($gameSwitches._lucilleKidnapped) {
      this.addCommand(TextManager.fight, 'lucille', $gameParty.lucilleEnergy >= 80 && ConfigManager.lucilleDifficulty !== 3);
    }
  };
};

Scene_Battle.prototype.createPartyCommandWindow = function() {
    this._partyCommandWindow = new Window_PartyCommand();
    this._partyCommandWindow.setHandler('lucille', this.commandCallLucille.bind(this));
    this._partyCommandWindow.setHandler('fight',  this.commandFight.bind(this));
    this._partyCommandWindow.setHandler('escape', this.commandEscape.bind(this));
    this._partyCommandWindow.deselect();
    this.addWindow(this._partyCommandWindow);
};

Game_Party.prototype.initialize = function() {
  old_gameparty_init.call(this);
    this.lucilleEnergy = 0;
    this.maxLucilleEnergy = 200;
}

BattleManager.startBattle = function() {
  // Run Original Function
  old_battlemanager_start.call(this);
  $gameParty.maxLucilleEnergy = Macarona.setMaxEnergy();
  // Always start with half of your energy
  $gameParty.lucilleEnergy = Math.floor($gameParty.maxLucilleEnergy / 2);
  $gameTemp._callTurnCount = 0;
};

Game_Action.prototype.apply = function(target) {
  old_gameaction_apply.call(this, target);
  let result = target.result();
  // Check iff hit
  let hit = result.isHit();
  if (hit && result.hpDamage > 0) {
    $gameParty.lucilleEnergy += Math.floor($gameParty.maxLucilleEnergy * 0.07);
  }
}

Object.defineProperty(Game_Party.prototype, 'lucilleEnergy', {
  get: function() {return this._lucilleEnergy;},
  set: function (value) {
    let maxEnergy = this._maxLucilleEnergy || 200;
    this._lucilleEnergy = value.clamp(0, maxEnergy);
  },
  configurable: true
});

Object.defineProperty(Game_Party.prototype, 'maxLucilleEnergy', {
  get: function() {return this._maxLucilleEnergy;},
  set: function (value) {
    this._maxLucilleEnergy = value.clamp(200, 400);
  },
  configurable: true
});

Macarona.setMaxEnergy = function() {
  let energyAmount = 200;
  // +4 Energy for every dialogue
  energyAmount += (Macarona.ModData.readData("dialogue_count").length * 4);
  // +2 Energy for every support skill used
  energyAmount += (Macarona.ModData.readData("support_count") * 2);
  return energyAmount;
}

function Window_LucilleStatus() { this.initialize.apply(this, arguments); }
Window_LucilleStatus.prototype = Object.create(Window_Base.prototype);
Window_LucilleStatus.prototype.constructor = Window_LucilleStatus;

//=============================================================================
// * Initialize Object
//=============================================================================
Window_LucilleStatus.prototype.initialize = function(x, y) {
  this._homePosition = new Point(x, y);
  this.portraitIndex = 0;
  this._energyAnim = {current: -2, target: 0, old: -1, duration: 0 };
  Window_Base.prototype.initialize.call(this, 14, 100, this.windowWidth(), this.windowHeight());
  this.opacity = 0;
  this.createSprites();
  this.refresh();
};

Window_LucilleStatus.prototype.standardPadding = function() { return 0; };
Window_LucilleStatus.prototype.windowWidth = function() { return 114; };
Window_LucilleStatus.prototype.windowHeight = function() { return 264; };

Window_LucilleStatus.prototype.refresh = function() {
  this.setStatusBack(this.chooseStatusBack(), false);
};

Window_LucilleStatus.prototype.update = function() {
    Window_Base.prototype.update.call(this);
    this.updatePortrait();
    this.updateFrameAnimations();
    this.updateBars();
}

const old_partywindow_update = Window_PartyCommand.prototype.update;
Window_PartyCommand.prototype.update = function() {
  old_partywindow_update.call(this);
  if ($gameSwitches._lucilleKidnapped) {
    this.updateFrameAnimations();
  }
}

Window_LucilleStatus.prototype.initFrameAnimations = function() {
    if (this._lucilleFace) {
        // Initialize Frame Animations
        this._frameAnimations = [
            {sprite: this._lucilleFace, rect: new Rectangle(0, 0, 106, 106), frames: [0, 1, 2], frameIndex: 0, delayCount:  0, delay: 12, active: true}
        ]
        this.updateFrameAnimations();
    }
};

Window_LucilleStatus.prototype.updateFrameAnimations = function() {
  // Go Through Animations
  for (var i = 0; i < this._frameAnimations.length; i++) {
    // Get Animation
    let index = this.portraitIndex;
    var anim = this._frameAnimations[i];
    // If Animation is active
    if (anim.active) {
      // If Animation Delay count is 0 or less
      if (anim.delayCount <= 0) {
        // Get Rectangle
        var rect = anim.rect;
        // Increase Current Frame
        anim.frameIndex = (anim.frameIndex + 1) % anim.frames.length;
        // Get Frame
        var frame = anim.frames[anim.frameIndex]
        // Set Animation Sprite Frame
        anim.sprite.setFrame(frame * rect.width, index * rect.height, rect.width, rect.height);
        // Reset Delay Count
        anim.delayCount = anim.delay;
      } else {
        // Decrease Delay Count
        anim.delayCount--
      };
    };
  };
};

Window_LucilleStatus.prototype.chooseStatusBack = function() {
  let index = 0;
  if (($gameTroop.turnCount() - $gameTemp._callTurnCount) > 2 && $gameParty._lucilleEnergy > 80 && !BattleManager._victoryPhase) {
    index = 2;
  } else {
    index = 0;
  }
  if ($gameParty.isAllDead() || SceneManager.isNextScene(Scene_Gameover)) {
    index = 1;
  }
  return index;
}

Window_LucilleStatus.prototype.setStatusBack = function(index, fade = true) {
  // If Fade flag
  if (fade) {
    // Create Sprite
    var sprite = new Sprite(ImageManager.loadSystem('more_faceset_states'));
    // Get Status Back Frame
    var frame = this._statusBackSprite._frame;
    // Set Sprite Frame
    sprite.setFrame(frame.x, frame.y, frame.width, frame.height);
    // Add Sprite to Status Back Sprite
    this._statusBackSprite.addChildAt(sprite, 0)
  };
  var width = 100, height = 100;
  var sx = (index % 4) * height;
  var sy = Math.floor(index / 4) * width;
  this._statusBackSprite.setFrame(sx, sy, width, height);
};

Window_PartyCommand.prototype.updateFrameAnimations = function() {
  // Go Through Animations
  for (var i = 0; i < this._frameAnimations.length; i++) {
    // Get Animation
    var anim = this._frameAnimations[i];
    // If Animation is active
    if (anim.active) {
      // If Animation Delay count is 0 or less
      if (anim.delayCount <= 0) {
        // Get Rectangle
        var rect = anim.rect;
        // Increase Current Frame
        anim.frameIndex = (anim.frameIndex + 1) % anim.frames.length;
        // Get Frame
        var frame = anim.frames[anim.frameIndex]
        // Set Animation Sprite Frame
        anim.sprite.setFrame(frame * rect.width, rect.y, rect.width, rect.height);
        // Reset Delay Count
        anim.delayCount = anim.delay;
      } else {
        // Decrease Delay Count
        anim.delayCount--
      };
    };
  };
};

const old_battle_createAllWindows = Scene_Battle.prototype.createAllWindows;
const old_battle_startPartyCommandSelection = Scene_Battle.prototype.startPartyCommandSelection;

Scene_Battle.prototype.createAllWindows = function() {
  // Run Original Function
  old_battle_createAllWindows.call(this);
  if ($gameSwitches._lucilleKidnapped && $gameParty.members().length < 5) {
    this.createLucilleWindow();
  }
};

Scene_Battle.prototype.createLucilleWindow = function() {
    this._lucilleStatusWindow = new Sprite();
    this.addChild(this._lucilleStatusWindow);
    this._lucilleStatusWindow.x = 0;
    this._lucilleContainer = new Window_LucilleStatus(7, 58);
    this._lucilleContainer.x = -120;
    if (!this._faceWindows[0].actor() || $modLoader.knownMods.has("absentmind") || !this._faceWindows[1].actor()) {
      this._lucilleContainer.y -= 161;
    } else if (!this._faceWindows[2].actor() || !this._faceWindows[3].actor()) {
      this._lucilleContainer.y += 167;
    };

    if ((!this._faceWindows[1].actor() || !this._faceWindows[3].actor()) && (this._faceWindows[0].actor() && this._faceWindows[2].actor()) && !$modLoader.knownMods.has("absentmind")) {
      this._lucilleStatusWindow.x += 498;
      this._lucilleContainer.x = 140;
    }

    if ($modLoader.knownMods.has("absentmind") && !this._faceWindows[1].actor()) {
      this._lucilleStatusWindow.x += 658;
      this._lucilleContainer.x = 400;
    }

    this._lucilleStatusWindow.addChild(this._lucilleContainer);

}

Scene_Battle.prototype.startPartyCommandSelection = function() {
  // Run Original Function
  old_battle_startPartyCommandSelection.call(this);
  if ($gameSwitches._lucilleKidnapped && $gameParty.members().length < 5) {
    this.showLucilleWindow();
  }
};

Window_LucilleStatus.prototype.createSprites = function() {
    var pos = this._homePosition;

    // Create status Back Sprite
    this._statusBackSprite = new Sprite();
    this._statusBackSprite.bitmap = ImageManager.loadSystem('more_faceset_states')
    this._statusBackSprite.x = pos.x + 7;
    this._statusBackSprite.y = pos.y + 17;
    this.setStatusBack(0, false);
    this.addChildAt(this._statusBackSprite, 1);

    this._lucilleFace = new Sprite(ImageManager.loadFace('LUCILLE_BATTLE'));
    this._lucilleFace.anchor.set(0.5, 1);
    this._lucilleFace.x = pos.x + 56;
    this._lucilleFace.y = pos.y + 126;
    // this._lucilleFace.opacity = 255;
    this._lucilleFace.setFrame(0, 0, 0, 0);
    this.addChildAt(this._lucilleFace, 2);

    this.initFrameAnimations();

    if ($modLoader.knownMods.has("absentmind")) {
      this._lucBoxSprite = new Sprite(ImageManager.loadSystem("absent_lucille_box"));
      this._energyBarSprite = new Sprite(ImageManager.loadSystem('absent_lucille_gradients'));
    } else {
      this._lucBoxSprite = new Sprite(ImageManager.loadSystem('lucille_box'));
      this._energyBarSprite = new Sprite(ImageManager.loadSystem('lucille_gradients'));
    }

    this._lucBoxSprite.x = pos.x;
    this._lucBoxSprite.y = pos.y;
    this.addChildAt(this._lucBoxSprite, 3);

    this._energyBarSprite.x = pos.x + 28;
    this._energyBarSprite.y = pos.y + 127;
    this._energyBarSprite.setFrame(0, 0, 81, 12);
    this.addChildAt(this._energyBarSprite, 4);
};

Window_LucilleStatus.prototype.drawEnergy = function(energy, maxEnergy) {
  var pos = this._homePosition;
  this.contents.fontSize = LanguageManager.getMessageData("XX_BLUE.Window_OmoriBattleActorStatus").drawHP_contents_fontsize;
  var y = pos.y + 112;
  const loc_position = [0, y + 4];
  this.contents.clearRect(loc_position[0], eval(loc_position[1]), this.width, 24);
  var maxText = ' ' + maxEnergy
  var width = this.textWidth(maxText) + 11;
  this.drawText(energy, pos.x, y, this.width - width, 'right');
  this.drawText(maxText, pos.x, y, this.width - 10,'right');
  this.drawText('/', pos.x, y + 1, this.width - width + 5,'right');
  this.resetFontSettings();
};

Window_LucilleStatus.prototype.updateBars = function() {
  let currentEnergy = $gameParty.lucilleEnergy;
  let maxEnergy = $gameParty.maxLucilleEnergy;

  var anim = this._energyAnim;
  if (currentEnergy !== anim.old) {
    anim.target = currentEnergy;
    anim.old = currentEnergy;
    anim.duration = Math.abs(anim.current - anim.target).clamp(0, 30);;
  };

  if (this._energyAnim.duration > 0) {
    var anim = this._energyAnim;
    anim.current = (anim.current * (anim.duration - 1) + anim.target) / anim.duration;
    anim.duration--;
    this.drawEnergy(Math.round(anim.current), maxEnergy)
    var width = (anim.current / maxEnergy) * 81;
    this._energyBarSprite._frame.width = width;
    this._energyBarSprite._refresh();
  }
};

Window_LucilleStatus.prototype.updatePortrait = function() {
  let oldPortraitIndex = this.portraitIndex;
  if (($gameTroop.turnCount() - $gameTemp._callTurnCount) > 2 && $gameParty._lucilleEnergy > 80) {
    this.portraitIndex = 2;
  } else {
    this.portraitIndex = 0;
  }
  if (BattleManager._victoryPhase) {
    this.portraitIndex = 1;
  }
  if ($gameParty.isAllDead() || SceneManager.isNextScene(Scene_Gameover)) {
    this.portraitIndex = 3;
  }
  if (this.portraitIndex !== oldPortraitIndex) {
    this.refresh();
  }
};

Scene_Battle.prototype.showLucilleWindow = function () {
    var duration = 25
    var obj = this._lucilleContainer;
    data = {
      obj: obj, properties: ['x'],  from: {x: obj.x}, to: {x: obj._homePosition.x }, durations: {x: duration},
      easing: Object_Movement.easeOutCirc
    };
    this.move.startMove(data);
}

Scene_Battle.prototype.showLucilleWindowTest = function (pos) {
    var duration = 25
    var obj = this._lucilleContainer;
    data = {
      obj: obj, properties: ['x'],  from: {x: obj.x}, to: {x: pos}, durations: {x: duration},
      easing: Object_Movement.easeOutCirc
    };
    this.move.startMove(data);
}

Scene_Battle.prototype.hideLucilleWindow = function() {
    var duration = 25
      var obj = this._lucilleContainer;
    data = {
      obj: obj, properties: ['x'],  from: {x: obj.x}, to: {x: -obj.windowWidth() }, durations: {x: duration},
      easing: Object_Movement.easeInCirc
    };
    this.move.startMove(data);
}







const old_scenebattle_showpartycommand = Scene_Battle.prototype.showPartyCommand;
const old_scenebattle_onenemycancel = Scene_Battle.prototype.onEnemyCancel;
const old_scenebattle_onactorcancel = Scene_Battle.prototype.onActorCancel;
// These are overrides
Scene_Battle.prototype.showPartyCommand = function() {
  if (!$gameSwitches._lucilleKidnapped) {
    old_scenebattle_showpartycommand.call(this);
    return;
  }
  // Duration
  var duration = 15
  // Get Object
  var obj = this._stressBar;
  // Create Movement Data
  var data = {
  obj: obj, properties: ['y'],  from: {y: obj.y}, to: {y: Graphics.height - 140 - 50}, durations: {y: duration},
  easing: Object_Movement.easeOutCirc
  };
  this.move.startMove(data);

  // Get Object
  obj = this._partyCommandWindow;
  // Create Movement Data
  data = {
    obj: obj, properties: ['y'],  from: {y: obj.y}, to: {y: Graphics.height - 92 - 50}, durations: {y: duration},
    easing: Object_Movement.easeOutCirc
  };
  this.move.startMove(data);

  // Get Object
  obj = this._actorCommandWindow;
  // Create Movement Data
  data = {
    obj: obj, properties: ['y'],  from: {y: obj.y}, to: {y: Graphics.height - 92}, durations: {y: duration},
    easing: Object_Movement.easeOutCirc
  };
  this.move.startMove(data);
};

Scene_Battle.prototype.onActorCancel = function() {
  // Run Original Function
  old_scenebattle_onactorcancel.call(this);
  this.moveStressBar();
};

Scene_Battle.prototype.moveStressBar = function() {
  // Duration
  var duration = 15
  // Get Object
  var obj = this._stressBar;
  // Create Movement Data
  var data = {
    obj: obj, properties: ['y'],  from: {y: obj.y}, to: {y: Graphics.height - 140}, durations: {y: duration},
    easing: Object_Movement.easeOutCirc
  };
  this.move.startMove(data);
}
const old_battle_startactorcommand = Scene_Battle.prototype.startActorCommandSelection;

Scene_Battle.prototype.startActorCommandSelection = function() {
  // Run Original Function
  old_battle_startactorcommand.call(this);
  this.moveStressBar();
};

Window_PartyCommand.prototype.itemRect = function(index) {
  // Get Rect
  var rect = Window_Command.prototype.itemRect.call(this, index);
  if ($gameVariables.value(22) === 5) {
    rect.x += 95;
    rect.y += 20;
  } else {
    rect.x += [60, 80, 25][index];
    rect.y += [-5, 0, 10][index];
  };
  return rect;
};


// State Icons Override
let _old_window_battle_actor_status_updatePositions = Window_OmoriBattleActorStatus.prototype.updatePositions;
Window_OmoriBattleActorStatus.prototype.updatePositions = function() {
    _old_window_battle_actor_status_updatePositions.call(this, ...arguments);

    if (this._stateIcons && $gameSwitches._lucilleKidnapped && ($gameParty.members().length >= 4) && $modLoader.knownMods.has("reverie")) {
      this._stateIcons.x = (this.width - 96) / 2;
      if (this.x < 200) {
        this._stateIcons.y = 102;
      } else {
        this._stateIcons.y = this.y < 240 ? this.height : -16;
      }

      // Determine if we need to update
      let nc = [];
      let actor = this.actor();
      if (actor) {
          getNc(actor, nc);
      }
      if (JSON.stringify(nc) !== JSON.stringify(this._stateIconCache)) {
          this._stateIcons.bitmap.fillAll('rgba(0, 0, 0, 0)')
          while(this._stateIcons.children[0]) {
              this._stateIcons.removeChild(this._stateIcons.children[0]);
          }
          let totalLength = nc.length;
          let elapsed = 0;
          for (let [s, t] of nc) {
              icon = $dataStates[s].meta.StateIcon.trim();
              let sprite = new Sprite(ImageManager.loadSystem(icon));
              this._stateIcons.addChild(sprite);
              sprite.anchor.set(0, 0);
              sprite.y = 0;
              sprite.x = ((96 - (totalLength * 16)) / 2) + elapsed * 16;
              sprite._stateSpriteTurnsLeft = t;
              elapsed++;
          }

          this._stateIconCache = nc;
      }
      // Update alpha separately
      for (let sprite of this._stateIcons.children) {
          sprite.alpha = getAlpha(sprite);
      }
      this._stateIcons.update();
      //if ($gameScreen._brightness === 255) debugger;
    }
} 


// Challenge Mode Badge
let old_system_onbattlewin = Game_System.prototype.onBattleWin;
Game_System.prototype.onBattleWin = function() {
    old_system_onbattlewin.call(this);
    if ($gameTemp._callTurnCount > 0 && ConfigManager.lucilleDifficulty === 2) {
      DGT.UnlockBadge("lucilleeverywhere", "challenge");
    }
};
