
// This plugin mainly handles the dialogue, as well as a few extra functions

var Macarona = Macarona || {};

// HUNT DOWN AND KILL LUCILLE
Macarona.deleteLucille = function() {
  var mapId = $gameMap.mapId();
  var data = $gameSystem.getMapSpawnedEventData(mapId);
  var length = data.length;
  for (var i = 1; i < length; ++i) {
    var eventData = data[i];
    if (!eventData) continue;
    var eventId = eventData.eventId();
    if (eventData._characterName && eventData._characterName === "DW_AUBREY_FLOUR") {
      Yanfly.DespawnEventID(eventId);
    }
  }
};

// Fixes Lucille's position in the follower array
Macarona.lucilleShift = function(lucindex) {
  // Go Through Data
  for (var i = 0; i < $gamePlayer.followers()._data.length; i++) {
    var follower = $gamePlayer.followers()._data[i];
    if (follower._graphicsData !== undefined && follower._graphicsData.id === 'lucille') {
      delete follower._graphicsData;
      delete follower._characterMovementGraphics;
    };
  };
  $gamePlayer.refresh();
  let lucillePosition = Macarona.getLucillePosition();
  if (lucindex) {
    lucillePosition = lucindex;
  }
  $gamePlayer.followers().addExtraFollower('lucille', 'DW_AUBREY_FLOUR', 1, lucillePosition);
  // Add IDLE/WALK/RUN to MARI
  $gamePlayer.followers().follower(lucillePosition).setMovementGraphics({ name: 'DW_AUBREY_FLOUR', index: 1 }, 
  { name: 'DW_AUBREY_FLOUR', index: 1 }, { name: 'DW_AUBREY_FLOUR', index: 5 });
  $gamePlayer.refresh();
}

// Function overwrites for follower bits
Game_Followers.prototype.addExtraFollower = function(id, name, index, cIndex) {
  // Set default character index
  if (cIndex === undefined) { cIndex = this.nextExtraFollowerSlot(); };
  // Follower Object
  var follower = this._data[cIndex];
  if ($gameSwitches._lucilleKidnapped && follower._graphicsData !== undefined && follower._graphicsData.id === 'lucille') {
    delete follower._characterMovementGraphics;
    var lucilleDeleted = true;
  }
  // Set Graphics Data
  follower._graphicsData = {id: id, name: name, index: index};
  if (lucilleDeleted) {
    let newLucilleIndex = Macarona.getLucillePosition();
    this._data[newLucilleIndex]._graphicsData = {id: 'lucille', name: 'DW_AUBREY_FLOUR', index: '1'};
    this._data[newLucilleIndex]._characterMovementGraphics = {idle: { name: 'DW_AUBREY_FLOUR', index: 1 }, walking: { name: 'DW_AUBREY_FLOUR', index: 1 }, running: { name: 'DW_AUBREY_FLOUR', index: 5 }};
  }
  // Refresh
  this.refresh();
};

const old_follower_removeall = Game_Followers.prototype.clearExtraFollowers;
Game_Followers.prototype.clearExtraFollowers = function() {
  old_follower_removeall.call(this);
  if ($gameSwitches._lucilleKidnapped) {
    $gameSwitches._lucilleKidnapped = false;
    $gameMap.refresh();
  }
}

Macarona._oldAddActor = Game_Party.prototype.addActor;
Macarona._oldRemoveActor = Game_Party.prototype.removeActor;
Game_Party.prototype.addActor = function(actorId) {
  Macarona._oldAddActor.call(this, actorId);
  if ($gameSwitches._lucilleKidnapped) {
    Macarona.lucilleShift();
  }
};

Game_Party.prototype.removeActor = function(actorId) {
  Macarona._oldRemoveActor.call(this, actorId);
  if ($gameSwitches._lucilleKidnapped) {
    Macarona.lucilleShift();
  }
};

// Despawns Lucille event during save, and adds her when exiting save menu
Macarona._oldSaveMenuStart = Scene_OmoriFile.prototype.create;
Macarona._oldSaveMenuEnd = Scene_OmoriFile.prototype.terminate;
Scene_OmoriFile.prototype.create = function() {
  Macarona._oldSaveMenuStart.call(this);
  if ($gameMap._mapId && $gameMap._mapId > 0) {
    Yanfly.DespawnEventID($gameMap.LastSpawnedEventID());
  }
}

Scene_OmoriFile.prototype.terminate = function() {
  Macarona._oldSaveMenuEnd.call(this);
  if ($gameMap._mapId && $gameMap._mapId > 0) {
    $gameMap._lucilleActive = false;
  }
}

const old_mapUpdate = Game_Map.prototype.update;
const old_mapSetup = Game_Map.prototype.setup;

// Soawns Lucille on gamemap update
Game_Map.prototype.update = function(sceneActive) {
  old_mapUpdate.call(this, sceneActive);
  if (!this._lucilleActive) {
    if ($gameSwitches._lucilleKidnapped) {
      let makotoRNG = Math.randomInt(8);
      if (makotoRNG === 1) {
        Yanfly.SpawnEventRandomly(5, 16, false);
      }
      this._lucilleActive = true;
    } else {
      Yanfly.SpawnEventRandomly(5, 16, false);
      this._lucilleActive = true;
    }
  };
};

// Calls to respawn Lucille when map changes
Game_Map.prototype.setup = function(mapId) {
  old_mapSetup.call(this, mapId);
  this._lucilleActive = false;
}

// returns all points not inhabited by an event or collision
Game_Map.prototype.validLucilleSpawnPoints = function() {
    var data = [];
    var width = this.width();
    var height = this.height();
    for (var x = 0; x < width; ++x) {
      for (var y = 0; y < height; ++y) {
        if (this.eventsXy(x, y).length > 0) continue;
        if ($gamePlayer.x == x && $gamePlayer.y == y) continue;
        if (this.boat().posNt(x, y)) continue;
        if (this.ship().posNt(x, y)) continue;
        if (!this.isPassable(x, y, 2)) continue;
        data.push([x, y]);
      }
    }
    return data;
};

Yanfly.SpawnEventRandomly = function(mapId, eventId, preserved) {
    var data = $gameMap.validLucilleSpawnPoints();
    if (data.length <= 0) return;
    random = data[Math.floor(Math.random() * data.length)];
    Yanfly.SpawnEvent(mapId, eventId, random[0], random[1], preserved);

};

Game_Event.prototype.TeleportToRandom = function() {
  var data = $gameMap.validLucilleSpawnPoints();
  if (data.length <= 0) return;
  random = data[Math.floor(Math.random() * data.length)];
  this.locate(random[0], random[1]);
};


// Horrible Dialogue Code Incoming!

// Handles windowskin and text blips outside of macros! haha!
Macarona.lucilleWindow = function() {
  if (Imported.TH_WindowskinChange === 1) {
    ImageManager.loadSystem("window_lucille");
    $gameSystem.setWindowskin("window_lucille");
  }
};

Macarona.makotoWindow = function() {
  if (Imported.TH_WindowskinChange === 1) {
    ImageManager.loadSystem("window_makoto");
    $gameSystem.setWindowskin("window_makoto");
  }
  $gameSystem._msgSoundName = "dialogue_makoto_new"
};

Macarona.normalWindow = function() {
  if (Imported.TH_WindowskinChange === 1) {
    ImageManager.loadSystem("Window");
    $gameSystem.setWindowskin("Window");
  }
  $gameSystem._msgSoundName = "SYS_text"
}

// Main lucille Dialogue code
// This creates an array of message names in lucille_everywhere.yaml and picks randomly from them. The array is influenced by which mods are active and some other factors.
Macarona.playLucilleDialogue = function() {
  let dialogue_choices = []

  if (typeof $modLoader !== "undefined") {
    // Mod Dialogue Code
    if ($modLoader.knownMods.has('thedreamer')) {
      dialogue_choices.push("dreamer_1", "dreamer_3", "dreamer_5", "dreamer_6", "dreamer_8", "dreamer_11", "dreamer_14", "dreamer_15");
      switch ($gameMap._mapId) {
        // Per-Map overrides
        case 530:
          dialogue_choices = ["dreamer_35"];
          break;
        case 545: case 546: case 547:
          dialogue_choices = ["dreamer_30", "dreamer_31", "dreamer_33"];
          break;
        case 551: case 543: case 544: case 554:
          dialogue_choices = ["dreamer_17"];
          break;
        case 569: case 570:
          dialogue_choices = ["dreamer_40"];
          break;
        case 555:
          dialogue_choices = ["dreamer_23"];
          break;
        case 556: case 557:
          dialogue_choices = ["dreamer_25"];
          break;
        case 549: case 550: case 559:
          dialogue_choices = ["dreamer_38"];
          break;
        case 538: case 539: case 540: case 541:
          dialogue_choices = ["dreamer_26", "dreamer_28"];
          break;
        case 529:
          dialogue_choices = ["dreamer_37"];
          break;
        case 528: case 542: case 575:
          dialogue_choices = ["dreamer_36"];
          break;
        case 548: case 558:
          dialogue_choices = ["dreamer_18"];
          break;
        case 532: case 537: case 560:
          dialogue_choices = ["dreamer_21"];
          break;
        case 533: case 561: case 562: case 563: case 564: case 565: case 566: case 573:
          dialogue_choices = ["dreamer_42"];
          break;
        case 390: case 568:
          dialogue_choices = ["dreamer_43"];
          break;
        default:
          break;
      };
      if ($gameSwitches.value(75)) {
        // PIZZA TIME
        dialogue_choices = ["dreamer_41"];
      }
    };

    if ($modLoader.knownMods.has('omoriismissing')) {
      dialogue_choices.push("oim_3");
      switch ($gameVariables.value(46)) {
        // PER WORLD VALUE DIALOGUES
        case 0:
          dialogue_choices.push("oim_4", "oim_5", "oim_6");
          break;
        case 1:
          dialogue_choices.push("oim_7", "oim_8", "oim_9", "oim_10", "oim_11")
          break;
        case 2:
          dialogue_choices.push("oim_35", "oim_36", "oim_37", "oim_38")
          break;
        case 3:
          dialogue_choices.push("oim_23", "oim_24", "oim_25", "oim_26", "oim_27", "oim_28");
          if ($gameParty.members().contains($gameActors.actor(40))) {
            dialogue_choices.push("oim_40");
          }
          break;
        case 4:
          dialogue_choices.push("oim_30", "oim_31", "oim_32", "oim_33", "oim_41");
          break;
        default:
          break;
      }
      switch ($gameMap._mapId) {
        // Per-map Overrides
        // Reminder: Add Library key Dialogue
        case 623: case 624: case 625: case 626: case 627: case 628: case 629: case 630: case 631: case 632:
          dialogue_choices = ["oim_14"];
          break;
        case 637: case 638:
          dialogue_choices = ["oim_21"];
          break;
        case 639:
          dialogue_choices = ["oim_20"];
          break;
        case 640: case 641: case 642: case 643:
          dialogue_choices = ["oim_15", "oim_16"];
          break;
        case 647: case 648:
          dialogue_choices = ["oim_19"];
          break;
        case 649: case 650: case 651: case 652: case 653: case 654: case 655: case 656: case 657: case 658: case 659: case 660: case 661: case 662: case 663: case 664: case 665: case 666:
          dialogue_choices = ["oim_13"];
          break;
        case 667: case 668:
          dialogue_choices = ["oim_22"];
          break;
        case 711: case 712:
          dialogue_choices = ["oim_12"];
          break;
        case 718: case 719: case 720: case 721: case 722:
          dialogue_choices = ["oim_17", "oim_18"];
          break;
        case 728:
          dialogue_choices = ["oim_34"];
          break;
        case 730: case 731: case 732: case 733: case 734:
          dialogue_choices = ["oim_1"];
          break;
        case 735:
          dialogue_choices = ["oim_39"];
          break;
        case 736:
          dialogue_choices = ["oim_23", "oim_24", "oim_25", "oim_26", "oim_27", "oim_28"];
          break;
        case 737:
          dialogue_choices = ["oim_35", "oim_36", "oim_37", "oim_38"];
          break;
        case 738:
          dialogue_choices = ["oim_40"];
          break;
        case 739:
          dialogue_choices = ["oim_30", "oim_31", "oim_32", "oim_33"];
          break;
        default:
          break;
      };
    };

    if ($modLoader.knownMods.has('absentmind')) {
      dialogue_choices.push("absent_1", "absent_2", "absent_3", "absent_4", "absent_5", "absent_6", "absent_7", "absent_8", "absent_9", "absent_10", "absent_11");
    };

    if ($modLoader.knownMods.has('PinkLove')) {
      dialogue_choices.push("ptl_1", "ptl_2", "ptl_3", "ptl_4", "ptl_5", "ptl_6", "ptl_7");
    };

    if ($modLoader.knownMods.has('reverie')) {
      dialogue_choices.push("reverie_1", "reverie_3", "reverie_4", "reverie_5", "reverie_6");
      if ($gameParty.leader().actorId() === 18) {
        dialogue_choices = ["reverie_2"]
      }
      if ($dataEnemies[1242].name === "METATRON") {
        dialogue_choices = ["reverie_7"]
      }
    };

    if ($modLoader.knownMods.has('omoexe')) {
      dialogue_choices.push("exe_1", "exe_2", "exe_3", "exe_5", "exe_7", "exe_8");
      switch($gameMap._mapId) {
        case 520:
          dialogue_choices = ["exe_4"];
          break;
        case 587: case 589: case 590:
          dialogue_choices = ["exe_6"];
          break;
        default:
          break;
      }
    };

    if ($modLoader.knownMods.has('REWRITTEN')) {
      dialogue_choices.push("rewritten_1", "rewritten_2", "rewritten_3", "rewritten_4", "rewritten_5", "rewritten_6", "rewritten_7", "rewritten_8", "rewritten_9");
      if ([543, 544, 593, 594, 595, 596].contains($gameMap._mapId)) {
        dialogue_choices = ["rewritten_10"];
      }
    };

    if ($modLoader.knownMods.has('parallels')) {
      if ($gameVariables.value(22) === 1) {
        dialogue_choices.push("parallels_3", "parallels_4", "parallels_5");
      } else {
        dialogue_choices.push("parallels_1", "parallels_2");
      };
      if ($gameParty.members().contains($gameActors.actor(18))) {
        dialogue_choices.push("parallels_6");
      }
    };

    if ($modLoader.knownMods.has('volcaniapeaks') && $gameMap._mapId >= 514) {
      dialogue_choices.push("volcania_1", "volcania_2", "volcania_3", "volcania_4", "volcania_5");
    };

    if ($modLoader.knownMods.has('autumnbreak')) {
      dialogue_choices.push("autumn_1", "autumn_2", "autumn_3");
      if ($gameMap._mapId > 645 && $gameMap._mapId < 654) {
        dialogue_choices = ["autumn_4"];
      }
    };

    if ($modLoader.knownMods.has('cecilyspace_heaven') && $gameParty.leader().actorId() === 2) {
      dialogue_choices.push("cecily_1");
    };

    if ($modLoader.knownMods.has('omoribrokendreams')) {
      dialogue_choices.push("bd_1");
    };

    if ($modLoader.knownMods.has('brokendreams') || $modLoader.knownMods.has('roseglass')) {
      dialogue_choices.push("bd_2");
    };

    if ($modLoader.knownMods.has('MIH')) {
      dialogue_choices.push("mih_1", "mih_2", "mih_3", "mih_4");
    };

    if ($modLoader.knownMods.has('omoriboy')) {
      dialogue_choices.push("omoriboy_1", "omoriboy_2", "omoriboy_3", "omoriboy_4");
    };

    if ($modLoader.knownMods.has('AFTERWARDS')) {
      dialogue_choices.push("afterward_1", "afterward_2", "afterward_3", "afterward_4", "afterward_5");
    };

    if ($modLoader.knownMods.has('stupidocmod')) {
      dialogue_choices.push("oc_1", "oc_2", "oc_3", "oc_4");
    }

    if ($modLoader.knownMods.has('BSG')) {
      dialogue_choices.push("bsg_1", "bsg_2", "bsg_3", "bsg_4", "bsg_5");
    }

    if ($modLoader.knownMods.has('SunnyProject')) {
      dialogue_choices.push("sunny_1", "sunny_2", "sunny_3", "sunny_4");
    }

  };


  // Base Dialogue Code
  let dialogueRoll = 0;
  if (dialogue_choices.length > 1 && dialogue_choices.length < 5) {
    dialogueRoll = Math.randomInt(4);
  }

  if (dialogue_choices.length < 1 || dialogueRoll === 1) {
    dialogue_choices.push("message_1", "message_3", "message_5", "message_6", "message_8", "message_10", "message_16", "message_17", "message_19");
    if ($gameMap._mapId < 514) {
      switch ($gameVariables.value(22)) {
        case 1:
          dialogue_choices.push("message_24", "message_25", "message_26", "message_27");
          break;
        case 2:
          dialogue_choices = ["message_20", "message_21", "message_22", "message_23"];
          break;
      }
    }
  }
  let randomIndex = Math.floor(Math.random() * dialogue_choices.length);
  let final_dialogue_choice = dialogue_choices[randomIndex];
  Macarona._dialogueChosen = final_dialogue_choice;
  $gameMessage.showLanguageMessage(`lucille_everywhere.${final_dialogue_choice}`);
  Macarona.ModData.pushDialogueChoice(final_dialogue_choice);
}

// Lucille code, but for makoto
Macarona.playMakotoDialogue = function() {
  // Base Dialogue Code
  let dialogue_choices = [];
  if (typeof $modLoader !== "undefined") {
    if ($modLoader.knownMods.has('thedreamer')) {
      dialogue_choices.push("dreamer_1", "dreamer_2", "dreamer_3", "dreamer_4");
      if ([545, 546, 547].contains($gameMap._mapId)) {
        dialogue_choices = ["dreamer_5"]
      }
    };

    if ($modLoader.knownMods.has('omoriismissing')) {
      dialogue_choices.push("oim_1", "oim_2", "oim_3", "oim_4", "oim_5", "oim_6");
    };

    if ($modLoader.knownMods.has('PinkLove')) {
      dialogue_choices.push("ptl_1", "ptl_2", "ptl_3");
    };

    if ($modLoader.knownMods.has('omoexe')) {
      dialogue_choices.push("exe_1", "exe_2", "exe_3", "exe_4");
    };

    if ($modLoader.knownMods.has('parallels')) {
      if ($gameVariables.value(22) === 1) {
        dialogue_choices.push("parallels_2");
      } else {
        dialogue_choices.push("parallels_1", "parallels_3");
      }
    };

    if ($modLoader.knownMods.has('autumnbreak')) {
      dialogue_choices.push("autumn_1");
    };

    if ($modLoader.knownMods.has('volcaniapeaks') && $gameMap._mapId >= 514) {
      dialogue_choices.push("volcania_1");
    };

    if ($modLoader.knownMods.has('absentmind')) {
      dialogue_choices.push("absent_1", "absent_2", "absent_3", "absent_4", "absent_5");
    };

    if ($modLoader.knownMods.has('reverie')) {
      dialogue_choices.push("reverie_1", "reverie_2", "reverie_3");
      if ($gameParty.leader().actorId() === 18) {
        dialogue_choices = ["reverie_4"]
      }
    };

    if ($modLoader.knownMods.has('REWRITTEN')) {
      dialogue_choices.push("rewritten_1", "rewritten_2", "rewritten_3", "rewritten_4");
    };

    if ($modLoader.knownMods.has('MIH')) {
      dialogue_choices.push("mih_1", "mih_2", "mih_3");
    };

    if ($modLoader.knownMods.has('omoriboy')) {
      dialogue_choices.push("omoriboy_1", "omoriboy_2");
    };
  }


  // Base Dialogue Code
  let RNGDialogue = Math.randomInt(3);
  if (dialogue_choices.length < 1 || RNGDialogue == 1) {
    dialogue_choices.push("message_1", "message_2", "message_3", "message_4", "message_5", "message_6", "message_7", "message_9", "message_10", "message_11", "message_12");
  }
  if (dialogue_choices.length > 1) {
  // boywife
  dialogue_choices.push("message_8");
  }
  let randomIndex = Math.floor(Math.random() * dialogue_choices.length);
  let final_dialogue_choice = dialogue_choices[randomIndex];
  $gameMessage.showLanguageMessage(`makoto_everywhere.${final_dialogue_choice}`);
}

// Tracks some external variables and other bits for the sake of awarding badges
Macarona.badgeCheck = function() {
  // Badges
  if ($modLoader.knownMods.has('omoriismissing') && $gameSwitches.value(2299) && [720, 721].contains($gameMap._mapId)) {
    DGT.UnlockBadge("lucilleeverywhere", "idiot");
    $gameMap._interpreter.command115();
  };
  
  if ($modLoader.knownMods.has('omoriismissing') && [718, 719, 720, 721, 722].contains($gameMap._mapId)) {
    DGT.UnlockBadge("omoriismissing", "badge_LUC")
    $gameMap._interpreter.command115();
  }

  if ($modLoader.knownMods.has('omoriismissing') && [730, 731, 732, 733, 734, 735, 736, 737, 738, 739].contains($gameMap._mapId)) {
    $gameMap._interpreter.command115();
  }

  if (Macarona._dialogueChosen == "oim_40") {
    DGT.UnlockBadge("lucilleeverywhere", "besties");
  };

  if ($modLoader.knownMods.has('REWRITTEN') && $gameSwitches.value(1573)) {
    DGT.UnlockBadge("lucilleeverywhere", "idiot");
    $gameMap._interpreter.command115();
  }
  if ($modLoader.knownMods.has('omoexe') && [587, 589, 590].contains($gameMap._mapId)) {
    let aubreyId = 0;
    switch ($gameMap._mapId) {
      case 587:
        aubreyId = 9;
        break;
      case 589:
        aubreyId = 4;
        break;
      case 590:
        aubreyId = 6;
        break;
      default:
        break;
    }
    if (aubreyId > 0) {
      $gameMap.event(aubreyId).locate($gamePlayer.x, $gamePlayer.y - 1);
      $gameSwitches.setValue(1936, true);
      $gameVariables.setValue(77, 1);
      DGT.UnlockBadge("lucilleeverywhere", "idiot");
      $gameMap._interpreter.command115();
    }
  }
  if (Macarona._dialogueChosen == "dreamer_42") {
    DGT.UnlockBadge("lucilleeverywhere", "idiot");
    $gameMap._interpreter.command115();
  }
  if (Macarona._dialogueChosen == "cecily_1") {
    DGT.UnlockBadge("lucilleeverywhere", "cecily");
  }
  if (Macarona.ModData.readData("dialogue_count").length > 49) {
    DGT.UnlockBadge("lucilleeverywhere", "hardcore_fan");
  }
}

const old_title_screen_initialize = Scene_OmoriTitleScreen.prototype.initialize;
Scene_OmoriTitleScreen.prototype.initialize = function() {
  old_title_screen_initialize.call(this);
  ImageManager.reserveSystem("window_lucille",  0, this._imageReservationId);
  ImageManager.reserveSystem("window_makoto",  0, this._imageReservationId);
}

const very_old_window_base_prototype_update = Window_Base.prototype.update;
Window_Base.prototype.update = function() {
    very_old_window_base_prototype_update.call(this);
    this.updateLetterEffects();
};

// New Skill Functions

Macarona.pickLucilleSkill = function() {
  $gameTemp._lucilleActionChoice = "hp_heal";
  let partyArray = $gameParty.aliveMembers();
  let partyMembers = partyArray.length;
  var chargeCount = 0;
  var highHPCount = 0;
  var highMPCount = 0;
  var needsHeal = false;
  var needsJuice = false;
  for (i = 0; i < partyArray.length; i++) {
    if (partyArray[i].isStateAffected(33)) {
      chargeCount++;
    }
    let healthDecimal = partyArray[i].hp / partyArray[i].mhp;
    healthPercentage = healthDecimal.toFixed(2);
    if (healthPercentage < 0.35) {
      needsHeal = true;
    } else if (healthPercentage > 0.95) {
      highHPCount++;
    }
    let juiceDecimal = partyArray[i].mp / partyArray[i].mmp;
    juicePercentage = juiceDecimal.toFixed(2);
    if (juicePercentage < 0.35) {
      needsJuice = true;
    } else if (juicePercentage > 0.95) {
      highMPCount++;
    }
  }
  let choiceArray = ["hp_heal","mp_heal","buff"];
  // overrides
  if (needsHeal && needsJuice) {
    choiceArray = ["hp_heal", "mp_heal"];
  } else if (needsHeal) {
    choiceArray = ["hp_heal"];
  } else if (needsJuice) {
    choiceArray = ["mp_heal"];
  }
  // if everyone is healthy
  if (highHPCount === partyMembers && highMPCount === partyMembers) {
    choiceArray = ["buff"];
  }
  // if heals aren't up and oarty members aren't charged
  if (choiceArray.includes("buff") && chargeCount <= Math.ceil(partyArray.length / 2) && (ConfigManager.lucilleDifficulty === 0)) {
    choiceArray.push("charge");
  }

  // final rng
  $gameTemp._lucilleActionChoice = choiceArray[Math.floor(Math.random() * choiceArray.length)];

}

// these next 3 are unused but i dont feel like deleting them
Macarona.needHpHeal = function() {
  let partyArray = $gameParty.aliveMembers();
  for (i = 0; i < partyArray.length; i++) {
    let healthDecimal = partyArray[i].hp / partyArray[i].mhp;
    healthPercentage = healthDecimal.toFixed(2);
    if (healthPercentage < 0.35) {
      return true;
    }
  }
  return false;
};

Macarona.needMpHeal = function() {
  let partyArray = $gameParty.aliveMembers();
  for (i = 0; i < partyArray.length; i++) {
    let juiceDecimal = partyArray[i].mp / partyArray[i].mmp;
    juicePercentage = juiceDecimal.toFixed(2);
    if (juicePercentage < 0.35) {
      return true;
    }
  }
  return false;
};

Macarona.flirt = function() {
  let partyArray = $gameParty.aliveMembers();
  let chosenMember = partyArray[Math.floor(Math.random() * partyArray.length)];
  BattleManager._logWindow.push("addText", `LUCILLE... flirts with ${chosenMember.name()}?`)
  chosenMember.addState(Macarona._flirtStateId);
  BattleManager._logWindow.push("addText", `It was super effective!`)
}

// Creates the difficulty options in the settings menu
const old_datamanager_applydata = ConfigManager.applyData;
const old_makeoptionslist = Window_OmoMenuOptionsGeneral.prototype.makeOptionsList;
const old_datamanager_makedata = ConfigManager.makeData;
const old_processoptioncommand = Window_OmoMenuOptionsGeneral.prototype.processOptionCommand;

ConfigManager.lucilleDifficulty = 1;

Window_OmoMenuOptionsGeneral.prototype.processOptionCommand = function() {
  old_processoptioncommand.call(this);
  var index = this.index();
  var data = this._optionsList[index];
  // Switch Case Index
  switch (index) {
    case Macarona._optionIndex: ConfigManager.lucilleDifficulty = data.index; ;break;
  };
};


ConfigManager.makeData = function() {
  // Get Original Config
  var config = old_datamanager_makedata.call(this);
  // Set Config Settings
  config.lucilleDifficulty = this.lucilleDifficulty;
  // Return Config
  return config;
};

Window_OmoMenuOptionsGeneral.prototype.makeOptionsList = function() {
  old_makeoptionslist.call(this);
  this._optionsList.push({
      header: "LUCILLE RECRUIT DIFFICULTY:",
      options: [
          "EASY", "NORMAL", "CHALLENGE", "OFF"
      ],
      helpText: "Changes how strong LUCILLE's skills are. CHALLENGE includes debuffs.",
      spacing: 120,
      index: ConfigManager.lucilleDifficulty
  });
  Macarona._optionIndex = this._optionsList.length - 1;
};

ConfigManager.applyData = function(config) {
  // Run Original Function
  old_datamanager_applydata.call(this, config);
  this.lucilleDifficulty = config.lucilleDifficulty || 0;
};

// Lucille Button(s)
document.addEventListener('keydown', (e) => {
  if (SceneManager._scene instanceof Scene_Map) {
    if (typeof window.commands !== 'undefined' && window.commands.active) {
      return;
    }
    switch (e.key) {
      case "l":
        if ($gameSwitches._lucilleKidnapped) {
          Macarona.playLucilleDialogue();
          Macarona.badgeCheck();
          $gameMap._interpreter.setWaitMode('message');
        }
        return;
      case "k":
        // The K stands for Kill
        Macarona.deleteLucille();
        if ($gameSwitches._lucilleKidnapped) {
          $gamePlayer._followers.removeFollowerById('lucille');
          $gameSwitches._lucilleKidnapped = false;
        }
        return;
    }
  }
})